@@ -50,6 +50,7 @@ static int rc_initialized = 0;
 #define P_CODE     7
 #endif
 #define P_PIXELS   8
+#define P_NZINT    9
 
 #if LANG == JA
 #define CMT_HELPER	 "ӥ塼Խ"
@@ -60,9 +61,11 @@ static int rc_initialized = 0;
 #define CMT_SAVEHIST     "URL¸"
 #define CMT_KANJICODE    "ɽѴ"
 #define CMT_FRAME        "ե졼μưɽ"
+#define CMT_ARGV_IS_URL  "scheme Τʤ URL Ȥߤʤ"
 #define CMT_TSELF        "target̤ξ_selfѤ"
 #define CMT_DISPLINK     "μưɽ"
 #define CMT_MULTICOL     "ե̾Υޥɽ"
+#define CMT_ALT_ENTITY   "ƥƥ ASCII ɽɽ"
 #define CMT_COLOR        "顼ɽ"
 #define CMT_B_COLOR      "ʸο"
 #define CMT_A_COLOR      "󥫡ο"
@@ -80,6 +83,7 @@ static int rc_initialized = 0;
 #define CMT_FTP_PROXY    "FTPץ(URL)"
 #define CMT_NO_PROXY     "ץɥᥤ"
 #define CMT_NOPROXY_NETADDR	"ͥåȥɥ쥹ǥץΥå"
+#define CMT_NO_CACHE     "Cache Ȥʤ"
 #define CMT_DNS_ORDER	"̾ν"
 #define CMT_DROOT        "/ ɽǥ쥯ȥ(document root)"
 #define CMT_PDROOT       "/~user ɽǥ쥯ȥ"
@@ -97,6 +101,7 @@ static int rc_initialized = 0;
 #define CMT_USERAGENT    "User-Agent"
 #define CMT_ACCEPTLANG   "Ĥ(Accept-Language:)"
 #define CMT_DOCUMENTCODE "ʸʸ"
+#define CMT_SYSTEMCODE   "ƥʸ"
 #define CMT_WRAP         "ޤ֤"
 #define CMT_VIEW_UNSEENOBJECTS "طʲؤΥ󥯤"
 #ifdef __EMX__
@@ -109,6 +114,7 @@ static int rc_initialized = 0;
 #define CMT_IGNORE_NULL_IMG_ALT "IMG ALT°λ˥̾ɽ"
 #define CMT_IFILE        "ƥǥ쥯ȥΥǥåե"
 #define CMT_RETRY_HTTP   "URL˼ưŪ http:// 䤦"
+#define CMT_DECODE_CTE   "¸ Content-Transfer-Encoding ǥɤ"
 #ifdef MOUSE
 #define CMT_MOUSE         "ޥȤ"
 #define CMT_REVERSE_MOUSE "ޥΥɥåưդˤ"
@@ -136,6 +142,7 @@ static int rc_initialized = 0;
 #define CMT_COOKIE_ACCEPT_DOMAINS "åդɥᥤ"
 #endif
 
+#define CMT_FOLLOW_REDIRECTION "쥯Ȥβ"
 
 #else				/* LANG != JA */
 
@@ -148,9 +155,11 @@ static int rc_initialized = 0;
 #define CMT_SAVEHIST     "Save URL history"
 /* #define CMT_KANJICODE    "Display Kanji Code" */
 #define CMT_FRAME        "Automatic rendering of frame"
+#define CMT_ARGV_IS_URL  "Force argument without scheme to URL"
 #define CMT_TSELF        "use _self as default target"
 #define CMT_DISPLINK     "Automatic display of link URL"
 #define CMT_MULTICOL     "Multi-column output of file names"
+#define CMT_ALT_ENTITY   "Use alternate expression with ASCII for entity"
 #define CMT_COLOR        "Display with color"
 #define CMT_B_COLOR      "Color of normal character"
 #define CMT_A_COLOR      "Color of anchor"
@@ -168,6 +177,7 @@ static int rc_initialized = 0;
 #define CMT_FTP_PROXY    "URL of FTP proxy host"
 #define CMT_NO_PROXY     "Domains for direct access (no proxy)"
 #define CMT_NOPROXY_NETADDR	"Check noproxy by network address"
+#define CMT_NO_CACHE     "Don't use cache"
 #define CMT_DNS_ORDER	"Order of name resolution"
 #define CMT_DROOT        "Directory corresponds to / (document root)"
 #define CMT_PDROOT       "Directory corresponds to /~user"
@@ -185,6 +195,7 @@ static int rc_initialized = 0;
 #define CMT_USERAGENT    "User-Agent"
 #define CMT_ACCEPTLANG   "Accept-Language"
 /* #define CMT_DOCUMENTCODE "Document Charset" */
+/* #define CMT_SYSTEMCODE   "System Kanji Code" */
 #define CMT_WRAP         "Wrap search"
 #define CMT_VIEW_UNSEENOBJECTS "Display unseenobjects (e.g. bgimage) tag"
 #ifdef __EMX__
@@ -197,6 +208,7 @@ static int rc_initialized = 0;
 #define CMT_IGNORE_NULL_IMG_ALT	"Ignore IMG ALT=\"\" (display link name)"
 #define CMT_IFILE        "Index file for the directory"
 #define CMT_RETRY_HTTP   "Prepend http:// to URL automatically"
+#define CMT_DECODE_CTE   "Decode Content-Transfer-Encoding when saving"
 #ifdef MOUSE
 #define CMT_MOUSE         "Use mouse"
 #define CMT_REVERSE_MOUSE "Reverse mouse dragging action"
@@ -223,6 +235,7 @@ static int rc_initialized = 0;
 #define CMT_COOKIE_REJECT_DOMAINS "Domains from which should reject cookies"
 #define CMT_COOKIE_ACCEPT_DOMAINS "Domains from which should accept cookies"
 #endif
+#define CMT_FOLLOW_REDIRECTION "Follow this number of redirections"
 #endif				/* LANG != JA */
 
 #define PI_TEXT    0
@@ -255,6 +268,13 @@ static struct sel_c dcodestr[] =
     {CODE_INNER_EUC, "I", STR_INNER_EUC},
     {0, NULL, NULL}
 };
+
+static struct sel_c scodestr[] =
+{
+    {CODE_EUC,       "E", STR_EUC},
+    {CODE_SJIS,      "S", STR_SJIS},
+    {0, NULL, NULL}
+};
 #endif				/* JP_CHARSET */
 
 #ifdef COLOR
@@ -309,18 +329,25 @@ static struct sel_c badcookiestr[] = {
 
 struct param_ptr params1[] =
 {
-    {"tabstop", P_INT, PI_TEXT, (void *) &Tabstop, CMT_TABSTOP, NULL},
+    {"tabstop", P_NZINT, PI_TEXT, (void *) &Tabstop, CMT_TABSTOP, NULL},
     {"pixel_per_char", P_PIXELS, PI_TEXT, (void *) &pixel_per_char, CMT_PIXEL_PER_CHAR, NULL},
 #ifdef JP_CHARSET
     {"kanjicode", P_CODE, PI_SEL_C, (void *) &DisplayCode, CMT_KANJICODE, kcodestr},
+    {"document_code", P_CODE, PI_SEL_C, (void *) &DocumentCode, CMT_DOCUMENTCODE, dcodestr},
+    {"system_code", P_CODE, PI_SEL_C, (void *) &SystemCode, CMT_SYSTEMCODE, scodestr},
 #endif				/* JP_CHARSET */
-  {"frame", P_CHARINT, PI_ONOFF, (void *) &RenderFrame, CMT_FRAME, NULL},
+    {"frame", P_CHARINT, PI_ONOFF, (void *) &RenderFrame, CMT_FRAME, NULL},
     {"target_self", P_CHARINT, PI_ONOFF, (void *) &TargetSelf, CMT_TSELF, NULL},
     {"display_link", P_INT, PI_ONOFF, (void *) &displayLink, CMT_DISPLINK, NULL},
     {"ext_dirlist", P_INT, PI_ONOFF, (void *) &UseExternalDirBuffer, CMT_EXT_DIRLIST, NULL},
     {"dirlist_cmd", P_STRING, PI_TEXT, (void *) &DirBufferCommand, CMT_DIRLIST_CMD, NULL},
-{"multicol", P_INT, PI_ONOFF, (void *) &multicolList, CMT_MULTICOL, NULL},
+    {"multicol", P_INT, PI_ONOFF, (void *) &multicolList, CMT_MULTICOL, NULL},
+    {"alt_entity", P_CHARINT, PI_ONOFF, (void *) &UseAltEntity, CMT_ALT_ENTITY, NULL},
     {"ignore_null_img_alt", P_INT, PI_ONOFF, (void *) &ignore_null_img_alt, CMT_IGNORE_NULL_IMG_ALT, NULL},
+#ifdef VIEW_UNSEENOBJECTS
+    {"view_unseenobject", P_INT, PI_ONOFF, (void *) &view_unseenobject, CMT_VIEW_UNSEENOBJECTS, NULL},
+#endif				/* VIEW_UNSEENOBJECTS */
+    {"show_lnum", P_INT, PI_ONOFF, (void *) &showLineNum, CMT_SHOW_NUM, NULL},
     {NULL, 0, 0, NULL, NULL, NULL},
 };
 
@@ -346,34 +373,20 @@ struct param_ptr params2[] =
 
 struct param_ptr params3[] =
 {
-  {"pagerline", P_INT, PI_TEXT, (void *) &PagerMax, CMT_PAGERLINE, NULL},
+    {"pagerline", P_NZINT, PI_TEXT, (void *) &PagerMax, CMT_PAGERLINE, NULL},
 #ifdef USE_HISTORY
-  {"history", P_INT, PI_TEXT, (void *) &URLHistSize, CMT_HISTSIZE, NULL},
-{"save_hist", P_INT, PI_ONOFF, (void *) &SaveURLHist, CMT_SAVEHIST, NULL},
+    {"history", P_INT, PI_TEXT, (void *) &URLHistSize, CMT_HISTSIZE, NULL},
+    {"save_hist", P_INT, PI_ONOFF, (void *) &SaveURLHist, CMT_SAVEHIST, NULL},
 #endif				/* USE_HISTORY */
     {"confirm_qq", P_INT, PI_ONOFF, (void *) &confirm_on_quit, CMT_CONFIRM_QQ, NULL},
-{"show_lnum", P_INT, PI_ONOFF, (void *) &showLineNum, CMT_SHOW_NUM, NULL},
-{"ftppasswd", P_STRING, PI_TEXT, (void *) &ftppasswd, CMT_FTPPASS, NULL},
-    {"user_agent", P_STRING, PI_TEXT, (void *) &UserAgent, CMT_USERAGENT, NULL},
-    {"no_referer", P_INT, PI_ONOFF, (void *) &NoSendReferer, CMT_NOSENDREFERER, NULL},
-    {"accept_language", P_STRING, PI_TEXT, (void *) &AcceptLang, CMT_ACCEPTLANG, NULL},
-#ifdef JP_CHARSET
-    {"document_code", P_CODE, PI_SEL_C, (void *) &DocumentCode, CMT_DOCUMENTCODE, dcodestr},
-#endif
- {"wrap_search", P_INT, PI_ONOFF, (void *) &WrapDefault, CMT_WRAP, NULL},
- {"ignorecase_search", P_INT, PI_ONOFF, (void *) &IgnoreCase, CMT_IGNORE_CASE, NULL},
-#ifdef VIEW_UNSEENOBJECTS
-    {"view_unseenobject", P_INT, PI_ONOFF, (void *) &view_unseenobject, CMT_VIEW_UNSEENOBJECTS, NULL},
-#endif				/* VIEW_UNSEENOBJECTS */
+    {"wrap_search", P_INT, PI_ONOFF, (void *) &WrapDefault, CMT_WRAP, NULL},
+    {"ignorecase_search", P_INT, PI_ONOFF, (void *) &IgnoreCase, CMT_IGNORE_CASE, NULL},
 #ifdef MOUSE
     {"use_mouse", P_INT, PI_ONOFF, (void *) &use_mouse, CMT_MOUSE, NULL},
     {"reverse_mouse", P_INT, PI_ONOFF, (void *) &reverse_mouse, CMT_REVERSE_MOUSE, NULL},
 #endif				/* MOUSE */
-    {"retry_http", P_INT, PI_ONOFF, (void *) &retryAsHttp, CMT_RETRY_HTTP, NULL},
     {"clear_buffer", P_INT, PI_ONOFF, (void *) &clear_buffer, CMT_CLEAR_BUF, NULL},
-#ifdef USE_SSL
-    {"ssl_forbid_method", P_STRING, PI_TEXT, (void *) &ssl_forbid_method, CMT_SSL_FORBID_METHOD, NULL},
-#endif				/* USE_SSL */
+    {"decode_cte", P_CHARINT, PI_ONOFF, (void *) &DecodeCTE, CMT_DECODE_CTE, NULL},
     {NULL, 0, 0, NULL, NULL, NULL},
 };
 
@@ -386,9 +399,8 @@ struct param_ptr params4[] =
     {"ftp_proxy", P_STRING, PI_TEXT, (void *) &FTP_proxy, CMT_FTP_PROXY, NULL},
     {"no_proxy", P_STRING, PI_TEXT, (void *) &NO_proxy, CMT_NO_PROXY, NULL},
     {"noproxy_netaddr", P_INT, PI_ONOFF, (void *) &NOproxy_netaddr, CMT_NOPROXY_NETADDR, NULL},
-#ifdef INET6
-    {"dns_order", P_INT, PI_SEL_C, (void *) &DNS_order, CMT_DNS_ORDER, dnsorders},
-#endif				/* INET6 */
+    {"no_cache", P_CHARINT, PI_ONOFF, (void *) &NoCache, CMT_NO_CACHE, NULL},
+
     {NULL, 0, 0, NULL, NULL, NULL},
 };
 
@@ -439,6 +451,23 @@ struct param_ptr params8[] =
     {NULL, 0, 0, NULL, NULL, NULL},
 };
 #endif
+struct param_ptr params9[] =
+{
+    {"ftppasswd", P_STRING, PI_TEXT, (void *) &ftppasswd, CMT_FTPPASS, NULL},
+    {"user_agent", P_STRING, PI_TEXT, (void *) &UserAgent, CMT_USERAGENT, NULL},
+    {"no_referer", P_INT, PI_ONOFF, (void *) &NoSendReferer, CMT_NOSENDREFERER, NULL},
+    {"accept_language", P_STRING, PI_TEXT, (void *) &AcceptLang, CMT_ACCEPTLANG, NULL},
+    {"argv_is_url", P_CHARINT, PI_ONOFF, (void *) &ArgvIsURL, CMT_ARGV_IS_URL, NULL},
+    {"retry_http", P_INT, PI_ONOFF, (void *) &retryAsHttp, CMT_RETRY_HTTP, NULL},
+    {"follow_redirection", P_INT, PI_TEXT, &FollowRedirection, CMT_FOLLOW_REDIRECTION, NULL},
+#ifdef USE_SSL
+    {"ssl_forbid_method", P_STRING, PI_TEXT, (void *) &ssl_forbid_method, CMT_SSL_FORBID_METHOD, NULL},
+#endif				/* USE_SSL */
+#ifdef INET6
+    {"dns_order", P_INT, PI_SEL_C, (void *) &DNS_order, CMT_DNS_ORDER, dnsorders},
+#endif				/* INET6 */
+    {NULL, 0, 0, NULL, NULL, NULL},
+};
 
 struct param_section sections[] =
 {
@@ -447,17 +476,15 @@ struct param_section sections[] =
 #ifdef COLOR
     {"ɽ", params2},
 #endif				/* COLOR */
-    {"¾", params3},
-    {"ץ", params4},
+    {"¿", params3},
     {"ǥ쥯ȥ", params5},
     {"ץ", params6},
+    {"ͥåȥ", params9},
+    {"ץ", params4},
 #if defined(USE_SSL) && defined(USE_SSL_VERIFY)
     {"SSLǧ", params7},
 #endif				/* defined(USE_SSL) &&
 				 * defined(USE_SSL_VERIFY) */
-#ifdef USE_SSL
-#define CMT_SSL_FORBID_METHOD "ȤʤSSL᥽åɤΥꥹ(2: SSLv2, 3: SSLv3, t:TLSv1)"
-#endif
 #ifdef USE_COOKIE
     {"å", params8},
 #endif
@@ -466,17 +493,15 @@ struct param_section sections[] =
 #ifdef COLOR
     {"Color Setting", params2},
 #endif				/* COLOR */
-    {"Other Behavior", params3},
-    {"Proxy Setting", params4},
+    {"Miscellaneous Setting", params3},
     {"Directory Setting", params5},
     {"External Programs", params6},
+    {"Network Setting", params9},
+    {"Proxy Setting", params4},
 #if defined(USE_SSL) && defined(USE_SSL_VERIFY)
     {"SSL Verification Setting", params7},
 #endif				/* defined(USE_SSL) &&
 				 * defined(USE_SSL_VERIFY) */
-#ifdef USE_SSL
-#define CMT_SSL_FORBID_METHOD N_("List of forbidden SSL method (2: SSLv2, 3: SSLv3, t:TLSv1)")
-#endif
 #ifdef USE_COOKIE
     {"Cookie Setting", params8},
 #endif
@@ -484,6 +509,8 @@ struct param_section sections[] =
     {NULL, NULL}
 };
 
+static Str to_str(struct param_ptr *p);
+
 static int
 compare_table(struct rc_search_table *a, struct rc_search_table *b)
 {
@@ -600,6 +627,7 @@ show_params(FILE * fp)
 	    case P_INT:
 	    case P_SHORT:
 	    case P_CHARINT:
+           case P_NZINT:
 		t = (sections[j].params[i].inputtype == PI_ONOFF) ? "bool" : "number";
 		break;
 	    case P_CHAR:
@@ -646,8 +674,8 @@ show_params(FILE * fp)
 }
 #endif
 
-static int
-str_to_bool(char *value)
+int
+str_to_bool(char *value, int old)
 {
     if (value == NULL)
 	return 1;
@@ -657,6 +685,18 @@ str_to_bool(char *value)
     case 'n':			/* no */
     case 'u':			/* undef */
 	return 0;
+    case 'o':
+       if (tolower(value[1]) == 'f')    /* off */
+           return 0;
+       return 1;        /* on */
+    case 't':
+       if (tolower(value[1]) == 'o')    /* toggle */
+           return ! old;
+       return 1;        /* true */
+    case '!':
+    case 'r':            /* reverse */
+    case 'x':            /* exchange */
+       return ! old;
     }
     return 1;
 }
@@ -774,13 +814,21 @@ set_param(char *name, char *value)
 	return 0;
     switch (p->type) {
     case P_INT:
-	*(int *) p->varptr = (p->inputtype == PI_ONOFF) ? str_to_bool(value) : atoi(value);
+       if (atoi(value) >= 0)
+           *(int *) p->varptr = (p->inputtype == PI_ONOFF)
+        ? str_to_bool(value, *(int *) p->varptr) : atoi(value);
+       break;
+    case P_NZINT:
+       if (atoi(value) > 0)
+           *(int *) p->varptr = atoi(value);
 	break;
     case P_SHORT:
-	*(short *) p->varptr = (p->inputtype == PI_ONOFF) ? str_to_bool(value) : atoi(value);
+       *(short *) p->varptr = (p->inputtype == PI_ONOFF)
+        ? str_to_bool(value, *(short *) p->varptr) : atoi(value);
 	break;
     case P_CHARINT:
-	*(char *) p->varptr = (p->inputtype == PI_ONOFF) ? str_to_bool(value) : atoi(value);
+       *(char *) p->varptr = (p->inputtype == PI_ONOFF)
+        ? str_to_bool(value, *(char *) p->varptr) : atoi(value);
 	break;
     case P_CHAR:
 	*(char *) p->varptr = value[0];
@@ -817,8 +865,6 @@ set_param(char *name, char *value)
     return 1;
 }
 
-static void sync_with_option(void);
-
 int
 set_param_option(char *option)
 {
@@ -851,10 +897,18 @@ set_param_option(char *option)
 	goto option_assigned;
     return 0;
  option_assigned:
-    sync_with_option();
     return 1;
 }
 
+char *
+get_param_option(char *name)
+{
+    struct param_ptr *p;
+
+    p = search_param(name);
+    return p ? to_str(p)->ptr : NULL;
+}
+
 static void
 interpret_rc(FILE * f)
 {
@@ -995,7 +1049,7 @@ initMimeTypes()
 	UserMimeTypes[i] = loadMimeTypes(tl->ptr);
 }
 
-static void
+void
 sync_with_option(void)
 {
     WrapSearch = WrapDefault;
@@ -1071,7 +1125,7 @@ init_rc(char *config_file)
 
 static char optionpanel_src1[] =
 "<html><head><title>Option Setting Panel</title></head>\
-<body><center><b>Option Setting Panel</b></center><p>\n"
+<body><center><b>Option Setting Panel</b><br><b>(w3m version %s)</b></center><p>\n"
 #ifdef __EMX__
 "<a href=\"file:///$LIB/w3mhelperpanel.exe?mode=panel\">%s</a>\n"
 #else				/* not __EMX__ */
@@ -1087,6 +1141,7 @@ to_str(struct param_ptr *p)
 #ifdef COLOR
     case P_COLOR:
 #endif
+    case P_NZINT:
 	return Sprintf("%d", *(int *) p->varptr);
     case P_SHORT:
 	return Sprintf("%d", *(short *) p->varptr);
@@ -1112,7 +1167,7 @@ to_str(struct param_ptr *p)
 Buffer *
 load_option_panel(void)
 {
-    Str src = Sprintf(optionpanel_src1, CMT_HELPER);
+    Str src = Sprintf(optionpanel_src1, version, CMT_HELPER);
     struct param_ptr *p;
     struct sel_c *s;
     int x, i;
@@ -1134,7 +1189,7 @@ load_option_panel(void)
 		Strcat_m_charp(src, "<input type=text name=",
 			       p->name,
 			       " value=\"",
-			       htmlquote_str( to_str(p)->ptr ),
+			       html_quote( to_str(p)->ptr ),
 			       "\">", NULL);
 		break;
 	    case PI_ONOFF:
