@@ -1,4 +1,4 @@
-/* $Id: table.h,v 1.4.2.9 2000/12/25 12:15:40 okabe Exp $ */
+/* $Id: table.h,v 1.3 2001/11/15 00:32:13 a-ito Exp $ */
 #if (defined(MESCHACH) && !defined(MATRIX))
 #define MATRIX
 #endif				/* (defined(MESCHACH) && !defined(MATRIX)) 
@@ -119,12 +119,13 @@ struct table {
 #define TBLM_NOBR 2
 #define TBLM_XMP 4
 #define TBLM_LST 8
+#define TBLM_PLAINTEXT 16
 #define TBLM_PRE_INT 32
 #define TBLM_INTXTA 64
 #define TBLM_INSELECT 128
-#define TBLM_PREMODE (TBLM_PRE|TBLM_INTXTA|TBLM_INSELECT|TBLM_XMP|TBLM_LST)
-#define TBLM_SPECIAL (TBLM_PRE|TBLM_PRE_INT|TBLM_XMP|TBLM_LST)
-#define TBLM_PLAIN (TBLM_XMP|TBLM_LST)
+#define TBLM_PREMODE (TBLM_PRE|TBLM_INTXTA|TBLM_INSELECT|TBLM_PLAIN)
+#define TBLM_SPECIAL (TBLM_PRE|TBLM_PRE_INT|TBLM_PLAIN)
+#define TBLM_PLAIN (TBLM_PLAINTEXT|TBLM_XMP|TBLM_LST)
 #define TBLM_SCRIPT 256
 #define TBLM_STYLE 512
 #define TBLM_IGNORE (TBLM_SCRIPT|TBLM_STYLE)
