@@ -54,7 +54,7 @@ static int
     21,				/* ftp */
     21,				/* ftpdir */
     0,				/* local - not defined */
-    0,				/* ??? - not defined? */
+    0,                /* local-CGI - not defined? */
     0,				/* exec - not defined? */
     119,			/* nntp */
     119,			/* news */
@@ -71,7 +71,7 @@ struct cmdtable schemetable[] =
     {"ftp", SCM_FTP},
     {"local", SCM_LOCAL},
     {"file", SCM_LOCAL},
-    {"exec", SCM_LOCAL_CGI},	/* ??? */
+/*  {"exec", SCM_EXEC}, */
     {"nntp", SCM_NNTP},
     {"news", SCM_NEWS},
     {"mailto", SCM_MAILTO},
@@ -276,7 +276,9 @@ openSSLHandle(int sock)
 #endif				/* defined(USE_SSL_VERIFY) */
 	    SSL_CTX_set_default_verify_paths(ssl_ctx);
 #endif				/* SSLEAY_VERSION_NUMBER >= 0x0800 */
-	atexit(free_ssl_ctx);
+/*** by inu
+        atexit(free_ssl_ctx);
+*/
     }
     handle = SSL_new(ssl_ctx);
     SSL_set_fd(handle, sock);
@@ -362,7 +364,7 @@ openSocket(char *hostname,
     MySignalHandler(*trap) ();
 
     if (fmInitialized) {
-        message(Sprintf("Opening socket...\n")->ptr, 0, 0);
+        message(Sprintf("Opening socket...")->ptr, 0, 0);
         refresh();
     }
     if (SETJMP(AbortLoading) != 0) {
@@ -452,7 +454,7 @@ openSocket(char *hostname,
 	hostaddr.sin_family = AF_INET;
 	hostaddr.sin_port = s_port;
 	if (fmInitialized) {
-	    message(Sprintf("Connecting to %s\n", hostname)->ptr, 0, 0);
+	    message(Sprintf("Connecting to %s", hostname)->ptr, 0, 0);
 	    refresh();
 	}
 	if (connect(sock, (struct sockaddr *) &hostaddr,
@@ -467,7 +469,7 @@ openSocket(char *hostname,
 	char **h_addr_list;
 	int result;
 	if (fmInitialized) {
-	    message(Sprintf("Performing hostname lookup on %s\n", hostname)->ptr, 0, 0);
+	    message(Sprintf("Performing hostname lookup on %s", hostname)->ptr, 0, 0);
 	    refresh();
 	}
 	if ((entry = gethostbyname(hostname)) == NULL) {
@@ -489,7 +491,7 @@ openSocket(char *hostname,
 		     adr & 0xff);
 #endif
 	    if (fmInitialized) {
-		message(Sprintf("Connecting to %s\n", hostname)->ptr, 0, 0);
+		message(Sprintf("Connecting to %s", hostname)->ptr, 0, 0);
 		refresh();
 	    }
 	    if ((result = connect(sock, (struct sockaddr *) &hostaddr,
@@ -557,9 +559,8 @@ parseURL(char *url, ParsedURL * p_url, ParsedURL * current)
 {
     char *p, *q;
     Str tmp;
-#ifdef __EMX__
-    int i;
-#endif
+
+    url = url_quote(url);	/* quote 0x01-0x20, 0x7F-0xFF */
 
     p = url;
     p_url->scheme = SCM_MISSING;
@@ -569,6 +570,8 @@ parseURL(char *url, ParsedURL * p_url, ParsedURL * current)
     p_url->host = NULL;
     p_url->is_nocache = 0;
     p_url->file = NULL;
+    p_url->real_file = NULL;
+    p_url->query = NULL;
     p_url->label = NULL;
 
     if (*url == '#') {		/* label only */
@@ -576,39 +579,48 @@ parseURL(char *url, ParsedURL * p_url, ParsedURL * current)
 	    copyParsedURL(p_url, current);
 	goto do_label;
     }
-#ifdef __EMX__
-    for (i = 17; i > 0; i -= 10)
-	if (!strncmp(url, "file://localhost/", i)) {
-	    p_url->scheme = SCM_LOCAL;
-	    p += i;
-	    url += i;
-	    break;
-	}
-    if (*url == '/' && IS_ALPHA(url[1]) && (url[2] == ':' || url[2] == '|')) {
-	++p;
-	++url;
+#if defined( __EMX__ ) || defined( __CYGWIN__ )
+    if (!strncmp(url, "file://localhost/", 17)) {
+       p_url->scheme = SCM_LOCAL;
+       p += 17 - 1;
+       url += 17 - 1;
     }
-    if (IS_ALPHA(*url) && (url[1] == ':' || url[1] == '|')) {
-	p_url->scheme = SCM_LOCAL;
-	p_url->file = p = allocStr(url, 0);
-	if (p[1] == '|')
-	    p[1] = ':';
-    }
-    if (p_url->scheme == SCM_LOCAL)
-	goto analyze_file;
 #endif
+#ifdef SUPPORT_DOS_DRIVE_PREFIX
+    if (IS_ALPHA(*p) && (p[1] == ':' || p[1] == '|')) {
+	p_url->scheme = SCM_LOCAL;
+	goto analyze_file;
+    }
+#endif         /* SUPPORT_DOS_DRIVE_PREFIX */
     /* search for scheme */
     p_url->scheme = getURLScheme(&p);
-    if (p_url->scheme == SCM_MISSING) {		/* scheme not found */
-	if (current)
-	    copyParsedURL(p_url, current);
-	else
-	    p_url->scheme = SCM_LOCAL;
-	p = url;
-	goto analyze_file;
+    if (p_url->scheme == SCM_MISSING) {
+       /* scheme part is not found in the url. This means either
+	  (a) the url is relative to the current or (b) the url
+	  denotes a filename (therefore the scheme is SCM_LOCAL).
+	*/
+       if (current) {
+	   copyParsedURL(p_url, current);
+	   if (p_url->scheme == SCM_LOCAL_CGI)
+	       p_url->scheme = SCM_LOCAL;
+	   /* label part and query part don't inherit */
+	   p_url->label = NULL;
+	   p_url->query = NULL;
+       } else
+	   p_url->scheme = SCM_LOCAL;
+       p = url;
+       if (!strncmp(p,"//",2)) {
+           /* URL begins with // */
+           /* it means that 'scheme:' is abbreviated */
+           p += 2;
+           goto analyze_url;
+       }
+       /* the url doesn't begin with '//' */
+       goto analyze_file;
     }
+    /* scheme part has been found */
     /* get host and port */
-    if (p[0] != '/' || p[1] != '/') {	/* scheme:foo or * scheme:/foo */
+    if (p[0] != '/' || p[1] != '/') {	/* scheme:foo or scheme:/foo */
 	p_url->host = NULL;
 	if (p_url->scheme != SCM_UNKNOWN)
 	    p_url->port = DefaultPort[p_url->scheme];
@@ -616,25 +628,29 @@ parseURL(char *url, ParsedURL * p_url, ParsedURL * current)
 	    p_url->port = 0;
 	goto analyze_file;
     }
+    /* after here, p begins with // */
     if (p_url->scheme == SCM_LOCAL) {	/* file://foo           */
 #ifdef __EMX__
 	p += 2;
 	goto analyze_file;
 #else
-	if (p[2] == '/' || p[2] == '~') {	/* file:///foo  */
-	    p += 2;		/* file://~user */
+        if (p[2] == '/' || p[2] == '~'
+        /* <A HREF="file:///foo">file:///foo</A>  or <A HREF="file://~user">file://~user</A> */
+#ifdef SUPPORT_DOS_DRIVE_PREFIX
+           || (IS_ALPHA(p[2]) && (p[3] == ':' || p[3] == '|'))
+           /* <A HREF="file://DRIVE/foo">file://DRIVE/foo</A> */
+#endif        /* SUPPORT_DOS_DRIVE_PREFIX */
+           ) {
+	    p += 2;		
 	    goto analyze_file;
 	}
-#endif
-#ifdef CYGWIN
-	goto analyze_file;	/* file://DRIVE/foo or *
-				 * file://machine/share/foo     */
-#endif				/* CYGWIN */
+#endif	/* __EMX__ */
     }
     p += 2;			/* scheme://foo         */
+	                        /*          ^p is here  */
   analyze_url:
     q = p;
-    while (*p && *p != ':' && *p != '/' && *p != '@') {
+    while (*p && strchr(":/@?#",*p) == NULL) {
 #ifdef INET6
 	if (*p == '[') {        /* rfc2732 compliance */
 	    char *p_colon = NULL;
@@ -656,45 +672,53 @@ parseURL(char *url, ParsedURL * p_url, ParsedURL * current)
 	p++;
     }
     switch (*p) {
-    case '\0':			/* scheme://host                */
-	/* scheme://user@host           */
-	/* scheme://user:pass@host      */
+    case '\0':
+	/* scheme://host                */
 	p_url->host = copyPath(q, -1, COPYPATH_SPC_IGNORE);
 	p_url->port = DefaultPort[p_url->scheme];
-	goto analyze_file;
+	p_url->file = DefaultFile(p_url->scheme);
+	return;
     case ':':
+	/* scheme://user:pass@host or
+	   scheme://host:port
+	 */
 	p_url->host = copyPath(q, p - q, COPYPATH_SPC_IGNORE);
 	q = ++p;
-	while (*p && *p != '/' && *p != '@')
+	while (*p && strchr("@/?#",*p) == NULL)
 	    p++;
-	if (*p == '@') {	/* scheme://user:pass@...       */
+	if (*p == '@') {
+	    /* scheme://user:pass@...       */
 	    p_url->pass = copyPath(q, p - q, COPYPATH_SPC_ALLOW);
 	    q = ++p;
 	    p_url->user = p_url->host;
 	    p_url->host = NULL;
 	    goto analyze_url;
 	}
+	/* scheme://host:port/ */
 	tmp = Strnew_charp_n(q, p - q);
 	p_url->port = atoi(tmp->ptr);
-	if (*p == '\0') {	/* scheme://host:port           */
+	if (*p == '\0') {
 	    /* scheme://user@host:port      */
 	    /* scheme://user:pass@host:port */
 	    p_url->file = DefaultFile(p_url->scheme);
 	    p_url->label = NULL;
 	    return;
 	}
+	/* *p is one of ['/', '?', '#'] */
 	break;
-    case '@':			/* scheme://user@...            */
+    case '@':
+	/* scheme://user@...            */
 	p_url->user = copyPath(q, p - q, COPYPATH_SPC_IGNORE);
 	q = ++p;
 	goto analyze_url;
-    case '/':			/* scheme://host/...            */
-	/* scheme://user@host/...       */
-	/* scheme://user:pass@host/...  */
+    case '/':
+    case '?':
+    case '#':
 	p_url->host = copyPath(q, p - q, COPYPATH_SPC_IGNORE);
 	p_url->port = DefaultPort[p_url->scheme];
 	break;
     }
+  analyze_file:
 #ifdef INET6
     /* rfc2732 compliance */
     if (p_url->host != NULL && p_url->host[0] == '[' &&
@@ -703,13 +727,38 @@ parseURL(char *url, ParsedURL * p_url, ParsedURL * current)
 	++(p_url->host);
     }
 #endif
-  analyze_file:
+#ifndef SUPPORT_NETBIOS_SHARE
     if (p_url->scheme == SCM_LOCAL && p_url->user == NULL &&
 	p_url->host != NULL && strcmp(p_url->host, "localhost")) {
+	/*
+	  In the environments other than CYGWIN, a URL like 
+	  file://host/file is regarded as ftp://host/file.
+	  On the other hand, file://host/file on CYGWIN is
+	  regarded as local access to the file //host/file.
+	  `host' is a netbios-hostname, drive, or any other
+	  name; It is CYGWIN system call who interprets that.
+	*/
+	  
 	p_url->scheme = SCM_FTP;	/* ftp://host/... */
 	if (p_url->port == 0)
 	    p_url->port = DefaultPort[SCM_FTP];
     }
+#endif
+#ifdef SUPPORT_DOS_DRIVE_PREFIX
+    if (p_url->scheme == SCM_LOCAL) {
+       q = p;
+       if (*q == '/')
+           q++;
+       if (IS_ALPHA(q[0]) && (q[1] == ':' || q[1] == '|')) {
+           if (q[1] == '|') {
+	       p = allocStr(q, 0);
+	       p[1] = ':';
+           }
+           else
+	       p = q;
+       }
+    }
+#endif
 
     q = p;
 #ifdef USE_GOPHER
@@ -744,7 +793,7 @@ parseURL(char *url, ParsedURL * p_url, ParsedURL * current)
     {
 	char *cgi = strchr(p, '?');
       again:
-	while (*p && *p != '#')
+	while (*p && *p != '#' && p != cgi)
 	    p++;
 	if (*p == '#' && p_url->scheme == SCM_LOCAL) {
 	    /* 
@@ -776,7 +825,14 @@ parseURL(char *url, ParsedURL * p_url, ParsedURL * current)
 	else
 	    p_url->file = copyPath(q, p - q, COPYPATH_SPC_IGNORE);
     }
-  do_label:
+ do_query:
+    if (*p == '?') {
+	q = ++p;
+	while (*p && *p != '#')
+	    p++;
+	p_url->query = copyPath(q,p-q,COPYPATH_SPC_ALLOW);
+    }
+ do_label:
     if (p_url->scheme == SCM_MISSING) {
 	p_url->scheme = SCM_LOCAL;
 	p_url->file = allocStr(p, 0);
@@ -789,6 +845,7 @@ parseURL(char *url, ParsedURL * p_url, ParsedURL * current)
 }
 
 #define initParsedURL(p) bzero(p,sizeof(ParsedURL))
+#define ALLOC_STR(s) ((s)==NULL?NULL:allocStr(s,0))
 
 void
 copyParsedURL(ParsedURL * p, ParsedURL * q)
@@ -796,26 +853,13 @@ copyParsedURL(ParsedURL * p, ParsedURL * q)
     p->scheme = q->scheme;
     p->port = q->port;
     p->is_nocache = q->is_nocache;
-    if (q->user)
-	p->user = allocStr(q->user, 0);
-    else
-	p->user = NULL;
-    if (q->pass)
-	p->pass = allocStr(q->pass, 0);
-    else
-	p->pass = NULL;
-    if (q->host)
-	p->host = allocStr(q->host, 0);
-    else
-	p->host = NULL;
-    if (q->file)
-	p->file = allocStr(q->file, 0);
-    else
-	p->file = NULL;
-    if (q->label)
-	p->label = allocStr(q->label, 0);
-    else
-	p->label = NULL;
+    p->user = ALLOC_STR(q->user);
+    p->pass = ALLOC_STR(q->pass);
+    p->host = ALLOC_STR(q->host);
+    p->file = ALLOC_STR(q->file);
+    p->real_file = ALLOC_STR(q->real_file);
+    p->label = ALLOC_STR(q->label);
+    p->query = ALLOC_STR(q->query);
 }
 
 void
@@ -823,6 +867,7 @@ parseURL2(char *url, ParsedURL * pu, ParsedURL * current)
 {
     char *p, *q;
     Str tmp;
+    int relative_uri = FALSE;
 
     parseURL(url, pu, current);
     if (pu->scheme == SCM_MAILTO)
@@ -832,6 +877,7 @@ parseURL2(char *url, ParsedURL * pu, ParsedURL * current)
 	pu->file = expandName(pu->file);
 
     if (current && pu->scheme == current->scheme) {
+	/* Copy omitted element from the current URL */
 	if (pu->user == NULL) {
 	    pu->user = current->user;
 	}
@@ -850,22 +896,22 @@ parseURL2(char *url, ParsedURL * pu, ParsedURL * current)
 		pu->scheme != SCM_NEWS &&
 #endif				/* USE_NNTP */
 		pu->file[0] != '/'
-#ifdef __EMX__
+#ifdef SUPPORT_DOS_DRIVE_PREFIX
 		&& !(pu->scheme == SCM_LOCAL && IS_ALPHA(pu->file[0]) && pu->file[1] == ':')
 #endif
 		) {
+		/* file is relative [process 1]*/
 		p = pu->file;
 		if (current->file) {
 		    tmp = Strnew_charp(current->file);
-		    if ((q = strchr(tmp->ptr, '?')) != NULL)
-			Strshrink(tmp, (tmp->ptr + tmp->length) - q);
 		    while (tmp->length > 0) {
 			if (Strlastchar(tmp) == '/')
 			    break;
 			Strshrink(tmp, 1);
 		    }
 		    Strcat_charp(tmp, p);
-		    pu->file = allocStr(tmp->ptr, 0);
+		    pu->file = tmp->ptr;
+		    relative_uri = TRUE;
 		}
 	    }
 #ifdef USE_GOPHER
@@ -880,6 +926,8 @@ parseURL2(char *url, ParsedURL * pu, ParsedURL * current)
 	    /* pu has only label */
 	    pu->file = current->file;
 	}
+	/* comment: query part need not to be completed
+	   from the current URL. */
     }
     if (pu->file) {
 #ifdef __EMX__
@@ -901,7 +949,11 @@ parseURL2(char *url, ParsedURL * pu, ParsedURL * current)
 	}
 #else
 	if (pu->scheme == SCM_LOCAL && pu->file[0] != '/' &&
+#ifdef SUPPORT_DOS_DRIVE_PREFIX /* for 'drive:' */
+           ! (IS_ALPHA(pu->file[0]) && pu->file[1] == ':') &&
+#endif
 	    strcmp(pu->file, "-")) {
+	    /* local file, relative path */
 	    tmp = Strnew_charp(CurrentDir);
 	    if (Strlastchar(tmp) != '/')
 		Strcat_char(tmp, '/');
@@ -909,15 +961,21 @@ parseURL2(char *url, ParsedURL * pu, ParsedURL * current)
 	    pu->file = cleanupName(tmp->ptr);
 	}
 #endif
-	else if ((pu->scheme == SCM_HTTP
+       else if (pu->scheme == SCM_HTTP
 #ifdef USE_SSL
 		|| pu->scheme == SCM_HTTPS
 #endif
-		) && pu->file[0] == '/') {
-		char *p = &pu->file[1];
-		int s = getURLScheme(&p);
-		if (s == SCM_MISSING || s == SCM_UNKNOWN)
-		    pu->file = cleanupName(pu->file);
+               ) {
+           if (relative_uri) {
+        /* In this case, pu->file is created by [process 1] above.
+           pu->file may contain relative path (for example, 
+           "/foo/../bar/./baz.html"), cleanupName() must be applied.
+           When the entire abs_path is given, it still may contain
+           elements like `//', `..' or `.' in the pu->file. It is 
+           server's responsibility to canonicalize such path.
+        */
+        pu->file = cleanupName(pu->file);
+           }
 	} else if (
 #ifdef USE_GOPHER
 		 pu->scheme != SCM_GOPHER &&
@@ -926,15 +984,34 @@ parseURL2(char *url, ParsedURL * pu, ParsedURL * current)
 		 pu->scheme != SCM_NEWS &&
 #endif				/* USE_NNTP */
 		 pu->file[0] == '/') {
+	    /*
+	      this happens on the following conditions:
+	      (1) ftp scheme (2) local, looks like absolute path.
+	      In both case, there must be no side effect with
+	      cleanupName(). (I hope so...)
+	    */
 	    pu->file = cleanupName(pu->file);
 	}
+	if (pu->scheme == SCM_LOCAL) {
+#ifdef SUPPORT_NETBIOS_SHARE
+	    if (pu->host && strcmp(pu->host,"localhost") != 0) {
+		Str tmp = Strnew_charp("//");
+        Strcat_m_charp(tmp, pu->host,
+                      cleanupName(file_unquote(pu->file)),
+			       NULL);
+		pu->real_file = tmp->ptr;
+	    }
+	    else
+#endif
+		pu->real_file = cleanupName(file_unquote(pu->file));
+	}
     }
 }
 
 static Str
 _parsedURL2Str(ParsedURL * pu, int pass)
 {
-    Str tmp = Strnew();
+    Str tmp;
     static char *scheme_str[] =
     {
 	"http", "gopher", "ftp", "ftp", "file", "file", "exec", "nntp", "news", "mailto",
@@ -989,13 +1066,17 @@ _parsedURL2Str(ParsedURL * pu, int pass)
 #ifdef USE_NNTP
 	pu->scheme != SCM_NEWS &&
 #endif				/* USE_NNTP */
-#ifdef __EMX__
-	(pu->file == NULL || (pu->file[0] != '/' && pu->file[1] != ':')))
-#else
-	(pu->file == NULL || pu->file[0] != '/'))
+       (pu->file == NULL || (pu->file[0] != '/'
+#ifdef SUPPORT_DOS_DRIVE_PREFIX
+       && !(IS_ALPHA(pu->file[0]) && pu->file[1] == ':' && pu->host == NULL)
 #endif
+       )))
 	Strcat_char(tmp, '/');
     Strcat_charp(tmp, pu->file);
+    if (pu->query) {
+	Strcat_char(tmp,'?');
+	Strcat_charp(tmp,pu->query);
+    }
     if (pu->label) {
 	Strcat_char(tmp, '#');
 	Strcat_charp(tmp, pu->label);
@@ -1043,7 +1124,7 @@ otherinfo(ParsedURL * target, ParsedURL * current, char *referer)
     else
 	Strcat_charp(s, UserAgent);
     Strcat_charp(s, "\r\n");
-    Strcat_charp(s, "Accept: text/*, image/*, audio/*, application/*\r\n");
+    Strcat_charp(s, "Accept: text/*, image/*, audio/*, video/*, application/*\r\n");
     Strcat_charp(s, "Accept-Encoding: gzip, compress, bzip, bzip2, deflate\r\n");
     Strcat_charp(s, "Accept-Language: ");
     if (AcceptLang != NULL && *AcceptLang != '\0') {
@@ -1064,7 +1145,7 @@ otherinfo(ParsedURL * target, ParsedURL * current, char *referer)
 	    Strcat(s, Sprintf(":%d", target->port));
 	Strcat_charp(s, "\r\n");
     }
-    if (target->is_nocache) {
+    if (target->is_nocache || NoCache) {
 	Strcat_charp(s, "Pragma: no-cache\r\n");
 	Strcat_charp(s, "Cache-control: no-cache\r\n");
     }
@@ -1113,6 +1194,10 @@ HTTPrequest(ParsedURL * pu, ParsedURL * current, HRequest * hr, TextList * extra
     }
     else if (hr->flag & HR_FLAG_LOCAL) {
 	Strcat_charp(tmp, pu->file);
+	if (pu->query) {
+	    Strcat_char(tmp,'?');
+	    Strcat_charp(tmp,pu->query);
+	}
     }
     else {
 	Strcat(tmp, _parsedURL2Str(pu, TRUE));
@@ -1145,15 +1230,22 @@ HTTPrequest(ParsedURL * pu, ParsedURL * current, HRequest * hr, TextList * extra
 	    Strcat_charp(tmp, "\r\n");
 	}
 	else {
-	    Strcat_charp(tmp, "Content-type: application/x-www-form-urlencoded\r\n");
+            if (!override_content_type) {
+               Strcat_charp(tmp, "Content-type: application/x-www-form-urlencoded\r\n");
+            }
 	    Strcat(tmp, Sprintf("Content-length: %ld\r\n", hr->request->length));
+            if (header_string)
+               Strcat (tmp, header_string);
 	    Strcat_charp(tmp, "\r\n");
-	    Strcat_charp(tmp, hr->request->body);
+           Strcat_charp_n(tmp, hr->request->body, hr->request->length);
 	    Strcat_charp(tmp, "\r\n");
 	}
     }
-    else
+    else {
+        if (header_string)
+           Strcat (tmp, header_string);
 	Strcat_charp(tmp, "\r\n");
+    }
 #ifdef DEBUG
     fprintf(stderr, "HTTPrequest: [ %s ]\n\n", tmp->ptr);
 #endif				/* DEBUG */
@@ -1183,9 +1275,9 @@ openURL(char *url, ParsedURL * pu, ParsedURL * current,
 	URLOption * option, FormList * request, TextList * extra_header,
 	URLFile * ouf, unsigned char *status)
 {
-    Str tmp = Strnew();
-    int sock;
-    char *p, *q;
+    Str tmp;
+    int i, sock, scheme;
+    char *p, *q, *u;
     URLFile uf;
     HRequest hr;
 #ifdef USE_SSL
@@ -1193,7 +1285,6 @@ openURL(char *url, ParsedURL * pu, ParsedURL * current,
 #endif				/* USE_SSL */
 #ifdef USE_NNTP
     FILE *fw;
-    int i;
     char *r;
     InputStream stream;
 #endif				/* USE_NNTP */
@@ -1205,14 +1296,21 @@ openURL(char *url, ParsedURL * pu, ParsedURL * current,
 	init_stream(&uf, SCM_MISSING, NULL);
     }
 
+    u = url;
+    scheme = getURLScheme(&u);
+    if (current == NULL && scheme == SCM_MISSING && ! ArgvIsURL)
+	u = file_to_url(url);		/* force to local file */
+    else
+	u = url;
   retry:
-    parseURL2(url, pu, current);
+    parseURL2(u, pu, current);
     if (pu->scheme == SCM_LOCAL && pu->file == NULL) {
 	if (pu->label != NULL) {
 	    /* #hogege is not a label but a filename */
 	    Str tmp2 = Strnew_charp("#");
 	    Strcat_charp(tmp2, pu->label);
 	    pu->file = tmp2->ptr;
+	    pu->real_file = cleanupName(file_unquote(pu->file));
 	    pu->label = NULL;
 	}
 	else {
@@ -1238,76 +1336,85 @@ openURL(char *url, ParsedURL * pu, ParsedURL * current,
     case SCM_LOCAL_CGI:
 	if (request && request->body) {
 	    /* local CGI: POST */
-	    uf.stream = newFileStream(localcgi_post(pu->file, request, option->referer),
+	    if ((q = strchr(pu->file, '?')) != NULL) {
+		p = Strnew_charp_n(pu->file, (int)(q - pu->file))->ptr;
+		pu->real_file = cleanupName(file_unquote(p));
+		q++;
+	    }
+	    uf.stream = newFileStream(localcgi_post(pu->real_file,
+						    pu->query,
+						    request,
+						    option->referer),
 				      (void (*)()) pclose);
 	    if (uf.stream == NULL)
 		goto ordinary_local_file;
 	    uf.is_cgi = TRUE;
-	    uf.scheme = SCM_LOCAL_CGI;
+	    uf.scheme = pu->scheme = SCM_LOCAL_CGI;
 	}
-	else if ((p = strchr(pu->file, '?')) != NULL) {
+	else if (pu->query != NULL) {
 	    /* lodal CGI: GET */
-	    for (q = pu->file; *q && *q != '?'; q++)
-		Strcat_char(tmp, *q);
-	    uf.stream = newFileStream(localcgi_get(tmp->ptr, p + 1, option->referer),
-					(void (*)()) pclose);
+	    uf.stream = newFileStream(localcgi_get(pu->real_file, pu->query,
+					option->referer), (void (*)()) pclose);
 	    if (uf.stream == NULL) {
-		pu->file = tmp->ptr;
+		pu->file = p;
 		goto ordinary_local_file;
 	    }
 	    uf.is_cgi = TRUE;
-	    uf.scheme = SCM_LOCAL_CGI;
+	    uf.scheme = pu->scheme = SCM_LOCAL_CGI;
 	}
+	else if ((i = strlen(pu->file)) > 4 &&
 #ifdef __EMX__
-	else if (!strncmp(pu->file + strlen(pu->file) - 4, ".cmd", 4))
+		!strncmp(pu->file + i - 4, ".cmd", 4))
 #else
-	else if (!strncmp(pu->file + strlen(pu->file) - 4, ".cgi", 4))
+		!strncmp(pu->file + i - 4, ".cgi", 4))
 #endif
 	{
 	    /* lodal CGI: GET */
-	    uf.stream = newFileStream(localcgi_get(pu->file, "", option->referer),
-				      (void (*)()) pclose);
+	    uf.stream = newFileStream(localcgi_get(pu->real_file, "",
+					option->referer), (void (*)()) pclose);
 	    if (uf.stream == NULL)
 		goto ordinary_local_file;
 	    uf.is_cgi = TRUE;
-	    uf.scheme = SCM_LOCAL_CGI;
+	    uf.scheme = pu->scheme = SCM_LOCAL_CGI;
 	}
 	else {
 	  ordinary_local_file:
-	    examineFile(pu->file, &uf);
+	    examineFile(pu->real_file, &uf);
 	}
 	if (uf.stream == NULL) {
-	    if (dir_exist(pu->file)) {
+	    if (dir_exist(pu->real_file)) {
 		add_index_file(pu, &uf);
 		if (uf.stream == NULL)
 		    return uf;
 	    }
 	    else if (document_root != NULL) {
-		Strcat_charp(tmp, document_root);
+		tmp = Strnew_charp(document_root);
 		if (Strlastchar(tmp) != '/' && pu->file[0] != '/')
 		    Strcat_char(tmp, '/');
 		Strcat_charp(tmp, pu->file);
-		tmp = Strnew_charp(cleanupName(tmp->ptr));
-		if (dir_exist(tmp->ptr)) {
-		    pu->file = tmp->ptr;
+		p = cleanupName(tmp->ptr);
+		q = cleanupName(file_unquote(p));
+		if (dir_exist(q)) {
+		    pu->file = p;
+		    pu->real_file = q;
 		    add_index_file(pu, &uf);
 		    if (uf.stream == NULL) {
 			return uf;
 		    }
 		}
 		else {
-		    examineFile(tmp->ptr, &uf);
-		    if (uf.stream)
-			pu->file = tmp->ptr;
+		    examineFile(q, &uf);
+		    if (uf.stream) {
+			pu->file = p;
+			pu->real_file = q;
+		    }
 		}
 	    }
 	}
 	if (uf.stream == NULL && retryAsHttp && url[0] != '/') {
-	    char *u = url;
-	    int scheme = getURLScheme(&u);
 	    if (scheme == SCM_MISSING || scheme == SCM_UNKNOWN) {
 		/* retry it as "http://" */
-		url = Strnew_m_charp("http://", url, NULL)->ptr;
+		u = Strnew_m_charp("http://", url, NULL)->ptr;
 		goto retry;
 	    }
 	}
@@ -1517,7 +1624,7 @@ openURL(char *url, ParsedURL * pu, ParsedURL * current,
 	    if (pu->file == NULL || *pu->file == '\0')
 		goto nntp_error;
 	    /* first char of pu->file is '/' */
-	    group = Strnew_charp(pu->file + 1)->ptr;
+	    group = url_unquote(Strnew_charp(pu->file + 1)->ptr);
 	    p = strchr(group, '/');
 	    if (p == NULL)
 		goto nntp_error;
@@ -1533,7 +1640,7 @@ openURL(char *url, ParsedURL * pu, ParsedURL * current,
 		goto nntp_error;
 	    fprintf(fw, "ARTICLE %s\r\n", p);
 	} else {
-	    fprintf(fw, "ARTICLE <%s>\r\n", pu->file);
+	    fprintf(fw, "ARTICLE <%s>\r\n", url_unquote(pu->file));
 	}
 	fflush(fw);
 	tmp = StrISgets(stream);
@@ -1562,18 +1669,20 @@ openURL(char *url, ParsedURL * pu, ParsedURL * current,
 static void
 add_index_file(ParsedURL * pu, URLFile *uf)
 {
-    Str tmp = Strnew_charp(pu->file);
+    char *p, *q;
 
     if (index_file == NULL || index_file[0] == '\0') {
 	uf->stream = NULL;
 	return;
     }
-    Strcat_m_charp(tmp, "/", index_file, NULL);
-    tmp->ptr = cleanupName(tmp->ptr);
-    examineFile(tmp->ptr, uf);
+    p = Strnew_m_charp(pu->file, "/", file_quote(index_file), NULL)->ptr;
+    p = cleanupName(p);
+    q = cleanupName(file_unquote(p));
+    examineFile(q, uf);
     if (uf->stream == NULL)
 	return;
-    pu->file = tmp->ptr;
+    pu->file = p;
+    pu->real_file = q;
     return;
 }
 
